//================================================================================================================
//
// The sky_render has an absolute spherical coordinate with it. The camera defines the origin of this coordinate.
//
//
//
// Constants which must be declared before including this file:
//
// float earth_r
// float atm_top_altitude
// float camera_altitude
// float sun_zenith_angle
// float sun_angular_diameter
//
//
//================================================================================================================

#declare sun_theta = radians(sun_zenith_angle);
#declare sun_steradian = 2*pi - 2*pi*cos(radians(sun_angular_diameter)/2);












//================================================================================================================
// Calculate the cosine between the sun light and another ray,
// input the absolute theta and the cosine of the absolute phi of the ray.
//================================================================================================================

#declare f_cos_between_direct_sun_light =
function(_the_theta, _the_phi_cos){
sin(_the_theta)*sin(sun_theta)*_the_phi_cos + cos(_the_theta)*cos(sun_theta)
}



//================================================================================================================
// Calculate the distance between two points,
// input the geocentric r of these two points and the relative zenith angle between them.
// Output unit is the same as the input r1 and r2, I always use kilometer.
//================================================================================================================

#declare f_dist_outward = // r1 < r2
function(_view_theta, _r1, _r2){
-_r1*cos(_view_theta) + sqrt(pow(_r1*cos(_view_theta),2) - _r1*_r1 + _r2*_r2)
}


#declare f_dist_inward = // r1 > r2
function(_view_theta, _r1, _r2){
-_r1*cos(_view_theta) - sqrt(pow(_r1*cos(_view_theta),2) - _r1*_r1 + _r2*_r2)
}



//================================================================================================================
// Calculate the altitude of a point,
// input the absolute r and the absolute theta of such a point. Output unit kilometer.
//================================================================================================================

#declare f_point_altitude =
function(_point_r, _point_theta){
sqrt(earth_r*earth_r + _point_r*_point_r + camera_altitude*camera_altitude + 2*earth_r*camera_altitude + 2*(earth_r+camera_altitude)*_point_r*cos(_point_theta)) - earth_r
}



//================================================================================================================
// Calculate the geocentric theta of a point,
// input the absolute r and the absolute theta of such a point. Output unit radians.
//================================================================================================================

#declare f_camera_theta_to_geoc_theta =
function(_point_r, _point_theta){
acos(
(_point_r*cos(_point_theta) + camera_altitude + earth_r)
/(f_point_altitude(_point_r, _point_theta) + earth_r)
)
}




//================================================================================================================
// Calculate the distance from a point in the atmosphere to a allocated altitude,
// input the absolute r, the absolute theta, the geocentric theta(to accelerate), and the absolute phi of this point,
// then input the absolute theta and the absolute phi of its view ray, then a target altitude. Output unit kilometer.
// Note that if the view ray touches the ground(altitude 0) before reaches the target altitude, it will return
// a negative sign distance from the start point to the ground touched point instead of the target altitude.
//================================================================================================================

#declare f_atmos_thck_from_point_internal = //calc angle OpointUkVqPviewVcos
function(_point_r, _point_geoc_r, _point_theta, _point_phi, _view_theta, _view_phi, calc_angle_cos, _to_geoc_r){
select(
_to_geoc_r - _point_geoc_r,
//Not available
0,
    select(
    -sqrt(_point_geoc_r*_point_geoc_r - earth_r*earth_r)/_point_geoc_r - calc_angle_cos,
    //To target altitude
    -_point_geoc_r*calc_angle_cos + sqrt(pow(_point_geoc_r*calc_angle_cos,2) - _point_geoc_r*_point_geoc_r + _to_geoc_r*_to_geoc_r),
    //To the ground
    _point_geoc_r*calc_angle_cos + sqrt(pow(_point_geoc_r*calc_angle_cos,2) - _point_geoc_r*_point_geoc_r + earth_r*earth_r)
    )
)
}

#declare f_atmos_thck_from_point = //interface
function(_point_r, _point_theta, _point_geoc_theta, _point_phi, _view_theta, _view_phi, _to_altitude){
f_atmos_thck_from_point_internal(
_point_r, earth_r + f_point_altitude(_point_r, _point_theta),
_point_theta, _point_phi,
_view_theta, _view_phi,
sin(_point_geoc_theta)*sin(_view_theta)*(cos(_point_phi)*cos(_view_phi)+sin(_point_phi)*sin(_view_phi)) + cos(_point_geoc_theta)*cos(_view_theta),
earth_r + _to_altitude
)
}





//================================================================================================================
// When calculate the secondary scattering, it needs add two vectors, the first one is (_r1, _theta1, _phi1), the
// second one is (_r2, _theta2, _phi2). All components here are the absolute spherical coordinate of the system.
// The f_secondary_calc_point_r() output is the r component of the sum of them. But the theta and the phi sum is
// more complex, so I add them to a rectangle coordinate vector temporary.
// That is the output of f_secondary_calc_point_x() and f_secondary_calc_point_z(). All output unit kilometer.
//================================================================================================================

#declare f_secondary_calc_point_x =  //W f_vector_sum_x, f_vector_sum_z, ...
function(_r1, _theta1, _phi1, _r2, _theta2, _phi2){
_r1*sin(_theta1)*cos(_phi1) + _r2*sin(_theta2)*cos(_phi2)
}
#declare f_secondary_calc_point_z =
function(_r1, _theta1, _r2, _theta2){
_r1*cos(_theta1) + _r2*cos(_theta2)
}
#declare f_secondary_calc_point_r =
function(_r1, _theta1, _phi1, _r2, _theta2, _phi2){
sqrt(_r1*_r1 + _r2*_r2 + 2*_r1*_r2*(sin(_theta1)*sin(_theta2)*(cos(_phi1)*cos(_phi2) + sin(_phi1)*sin(_phi2)) + cos(_theta1)*cos(_theta2)) )
}




#declare f_secondary_calc_sun_atmos_thck =
function(_point_r, _point_theta, _point_geoc_theta, _point_phi_cos){
f_atmos_thck_from_point_internal(
0, sqrt(earth_r*earth_r + _point_r*_point_r + camera_altitude*camera_altitude + 2*earth_r*camera_altitude + 2*(earth_r+camera_altitude)*_point_r*cos(_point_theta)),
0, 0,
0, 0,
sin(_point_geoc_theta)*sin(sun_theta)*_point_phi_cos + cos(_point_geoc_theta)*cos(sun_theta),
earth_r + atm_top_altitude
)
}
